'use strict';
var express = require('express'),
	router = express.Router(),
	returnCode = require('../../utils/returnCode'),
	_ = require('lodash');

var mongoose = require('mongoose');
var ForumInfo = mongoose.model('ForumInfo');
var forumInfoService=require('../../service/forumInfoService');
var forumThreadService=require('../../service/forumThreadService');


module.exports = function(app) {
	app.use('/admin/forum', router);
};

//新增论坛板块
router.post('/info/create', function(req, res, next) {
	var rs = {};
	req.body.ent_code=req.session.user.ent_code;
	forumInfoService.createInfo(req.body,function(err,info){
		if (err) {
			console.error(err);
			res.json(returnCode.BUSY);
		} else {
			rs.data = {'id':info._id};
			res.json(_.assign(rs, returnCode.SUCCESS));
		}
	});
});

//获取目标论坛板块
router.get('/info/:fid/get', function(req, res, next) {
	var fid = req.params.fid || null;
	var rs = {};
	if (fid) {
		forumInfoService.getInfoById(fid,function(err,info){
			if (err) {
				console.error(err);
				res.json(returnCode.BUSY);
			} else {
				rs.data = info;
				res.json(_.assign(rs, returnCode.SUCCESS));
			}
		});
	} else {
		res.json(returnCode.WRONG_PARAM);
	}
});

//更新目标论坛板块
router.post('/info/:fid/update', function(req, res, next) {
	var fid=req.params.fid;
	if(fid){
		forumInfoService.updateInfoById(fid,req.body,function(err,result){
			if(err){
				res.json(returnCode.BUSY);
			}else{
				res.json(returnCode.SUCCESS);
			}
		});
	}else{
		res.json(returnCode.WRONG_PARAM);
	}
});

//删除目标论坛板块
router.post('/info/:fid/delete', function(req, res, next) {
	var fid=req.params.fid;
	if(fid){
		forumInfoService.deleteInfoById(fid,function(err,result){
			if(err){
				res.json(returnCode.BUSY);
			}else{
				res.json(returnCode.SUCCESS);
			}
		});
	}else{
		res.json(returnCode.WRONG_PARAM);
	}
});

/**
 * [description]
 * @param  {[type]}
 * @param  {[type]}
 * @param  {[type]}
 * @return {[type]}
 */
router.get('/info/:fid/threads', function(req, res, next) {
	var pageNo = req.query.pageNo || 1;
    var pageSize = req.query.pageSize || 10;
    var fid=req.params.fid;

	if(fid){
		var conditions={
	    	ent_code:req.session.user.ent_code,
	    	info:fid
	    };
		forumThreadService.getAllThreadByFid(conditions,pageNo,pageSize,null,function(err,results){
			if(err){
				console.error(err);
				res.json(returnCode.BUSY);
			}else{
				res.json(_.assign(results, returnCode.SUCCESS));
			}
		});
	}else{
		res.json(returnCode.WRONG_PARAM);
	}
});

/**
 * [description]
 * @param  {[type]}
 * @param  {[type]}
 * @param  {[type]}
 * @return {[type]}
 */
router.get('/infos/list', function(req, res, next) {
	var pageNo = req.query.pageNo || 1;
    var pageSize = req.query.pageSize || 10;
    var groupId=req.query.groupId || null;
    var infoName=req.query.infoName || null;
    var conditions={
    	ent_code:req.session.user.ent_code
    };
    if(groupId){
    	conditions.group=groupId;
    }
	if(infoName){
    	conditions.name={
    		$regex: infoName,
			$options: 'i'
    	};
    }
	forumInfoService.getAllByGid(conditions,pageNo,pageSize,function(err,results){
		if(err){
			console.error(err);
			res.json(returnCode.BUSY);
		}else{
			res.json(_.assign(results, returnCode.SUCCESS));
		}
	});
});
