'use strict';
var express = require('express'),
    router = express.Router(),
    returnCode = require('../../utils/returnCode'),
    _ = require('lodash');
var mongoose = require('mongoose');
var ForumThread = mongoose.model('ForumThread');
var forumThreadService = require('../../service/forumThreadService');
var forumRolePermissionService = require('../../service/forumRolePermissionService');
var forumPraiseLogService = require('../../service/forumPraiseLogService');
var forumShareLogService = require('../../service/forumShareLogService');
var forumCommentService = require('../../service/forumCommentService');
var forumTagService = require('../../service/forumTagService');
var forumUserService = require('../../service/forumUserService');

var httpService = require('../../service/httpService');

var userUtil = require('../../utils/user');
var async = require('async');

module.exports = function(app) {
    app.use('/admin/forum', router);
};


//---------------------文章表单操作---------------------------------

//新增论坛文章
router.post('/thread/create', function(req, res, next) {

    var rs = {};
    req.body.ent_code = req.session.user.ent_code;

    var uid = req.session.user.id;
    // var uid='12345';
    if (req.body.pid) {
        req.body.level = 2;
    }
    async.waterfall([
        function(callback) {
            forumUserService.getUserByUid(uid, callback);
        }
    ], function(err, user) {
        if (err) {
            res.json(returnCode.BUSY);
        } else {
            if (user) {
                req.body.from = user._id;
                forumThreadService.createThread(req.body, function(err, entity) {
                    if (err) {
                        console.error(err);
                        res.json(returnCode.BUSY);
                    } else {
                        rs.data = {
                            'id': entity._id
                        };
                        res.json(_.assign(rs, returnCode.SUCCESS));
                    }
                });
            } else {
                var entity = {
                    uid: uid,
                    nickName: req.session.user.name,
                    icon: req.session.user.headPic
                };
                forumUserService.createUser(entity, function(err, doc) {
                    if (err) {
                        res.json(returnCode.BUSY);
                    } else {

                        req.body.from = doc._id;
                        forumThreadService.createThread(req.body, function(err, entity) {
                            if (err) {
                                console.error(err);
                                res.json(returnCode.BUSY);
                            } else {
                                rs.data = {
                                    'id': entity._id
                                };
                                res.json(_.assign(rs, returnCode.SUCCESS));
                            }
                        });
                    }
                }); } }
    });
});

//获取目标论坛文章
router.get('/thread/:tid/get', function(req, res, next) {
    var tid = req.params.tid || null;
    var rs = {};
    if (tid) {
        async.parallel([
            function(callback) {
                forumThreadService.getThreadById(tid, function(err, thread) {
                    if (err) {
                        callback(err, null);
                    } else {
                        callback(null, thread);
                    }
                });
            },
            function(callback) {
                forumTagService.getAllTag(req.session.user.ent_code, 1, 100, function(err, results) {
                    if (err) {
                        callback(err, null);
                    } else {
                        callback(null, results);
                    }
                });
            }
        ], function(err, results) {
            if (err) {
                res.json(returnCode.BUSY);
            } else {
                rs.data = results[0];
                rs.tagList = results[1];
                res.json(_.assign(rs, returnCode.SUCCESS));
            }
        });
    } else {
        res.json(returnCode.WRONG_PARAM);
    }
});

//更新文章状态、如：屏蔽
router.post('/thread/:tid/update', function(req, res, next) {
    var tid = req.params.tid;
    var fatherTitle = req.body.father;
    delete req.body.father;
    req.body.ent_code = req.session.user.ent_code;
    if (tid) {
        forumThreadService.updateThreadById(tid, req.body, function(err, thread) {
            if (err) {
                res.json(returnCode.BUSY);
            } else {
                res.json({
                    message: "success",
                    data: thread,
                    title: fatherTitle
                });
            }
        });
    } else {
        res.json(returnCode.WRONG_PARAM);
    }
});


//删除文章
router.post('/thread/:tid/delete', function(req, res, next) {
    var tid = req.params.tid;
    if (tid) {
        forumThreadService.deleteThreadById(tid, function(err, thread) {
            if (err) {
                res.json(returnCode.BUSY);
            } else {
                res.json(returnCode.SUCCESS);
            }
        });
    } else {
        res.json(returnCode.WRONG_PARAM);
    }
});

//文章置顶
router.post('/thread/:tid/:fid/top', function(req, res, next) {
    var tid = req.params.tid; //文章ID
    var fid = req.params.fid; //板块ID
    if (tid && fid) {
        forumThreadService.updateTopByThreadId(fid, tid, function(err, thread) {
            if (err) {
                res.json(returnCode.BUSY);
            } else {
                res.json(returnCode.SUCCESS);
            }
        });
    } else {
        res.json(returnCode.WRONG_PARAM);
    }
});

//文章取消置顶
router.post('/thread/:tid/:fid/unTop', function(req, res, next) {
    var tid = req.params.tid; //文章ID
    var fid = req.params.fid; //板块ID
    if (tid && fid) {
        forumThreadService.updateUnTopByThreadId(fid, tid, function(err, thread) {
            if (err) {
                res.json(returnCode.BUSY);
            } else {
                res.json(returnCode.SUCCESS);
            }
        });
    } else {
        res.json(returnCode.WRONG_PARAM);
    }
});

/**
 * [description]
 * @param  {[type]}
 * @param  {[type]}
 * @param  {[type]}
 * @return {[type]}
 */
router.get('/threads/list', function(req, res, next) {
    var pageNo = req.query.pageNo || 1;
    var pageSize = req.query.pageSize || 10;
    var infoId = req.query.infoId;
    var tagId = req.query.tagId;
    var pid = req.query.pid;
    var conditions = {
        ent_code: req.session.user.ent_code,
        level: 1
    };
    if (infoId) {
        conditions.info = infoId;
    }
    if (tagId) {
        conditions.tag = {
            $in: [tagId]
        };
    }

    if (pid) {
        conditions.pid = pid;
        conditions.level = 2;
    }

    forumThreadService.getAllThreadByFid(conditions, pageNo, pageSize, null, function(err, results) {
        if (err) {
            console.error(err);
            res.json(returnCode.BUSY);
        } else {
            res.json(_.assign(results, returnCode.SUCCESS));
        }
    });
});


//评论列表
router.get('/thread/:tid/comment/list', function(req, res, next) {

    var tid = req.params.tid || null;
    var pageNo = req.query.pageNo || 1;
    var pageSize = req.query.pageSize || 10;

    var conditions = {
        ent_code: req.session.user.ent_code,
        thread: tid,
        level: '1',
        status: {
            $ne: 2
        } //过滤被删除的数据，status 为 2 表示删除
    };

    if (tid) {
        //获取最新5条评论                                          
        forumCommentService.getAllComment(conditions, pageNo, pageSize, function(err, results) {
            if (err) {
                console.log(err);
                res.json(returnCode.BUSY);
            } else {
                res.json(_.assign(results, returnCode.SUCCESS));
            }
        });
    } else {
        res.json(returnCode.WRONG_PARAM);
    }
});

// 更新评论状态
router.post('/thread/comment/update/:cid', function(req, res, next) {
    var cid = req.params.cid || null;
    var status = req.body.status;
    var tid = req.body.tid;
    var comment_count = req.body.comment_count;
    var level = req.body.level;
    var parent_cid = req.body.parent_cid;
    if (cid) {
        forumCommentService.updateCommentStatusById(cid, status, function(err, result) {
            if (err) {
                res.json(returnCode.BUSY);
            } else {
                // 如果是删除
                if (status == 2) {
                    if (level == 1) { // 如果删除的是一级评论
                        // 获取评论的评论列表,并移除cid评论
                        forumThreadService.getById(tid, function(err, result) {
                            if (err) {
                                console.error(err);
                                callback(err, null);
                            } else {
                                result.comment_count -= 1;
                                var commentList = result.comments;
                                result.comments = forumThreadService.remove(commentList, cid);
                                forumThreadService.updateThreadById(tid, result, function(err, result) {
                                    if (err) {
                                        console.error(err);
                                        callback(err, null);
                                    } else {
                                        res.json(returnCode.SUCCESS);
                                    }
                                });
                            }
                        });
                    } else { // 删除子评论

                        // 更新文章评论 - 1
                        forumThreadService.updateThreadCommentCount(tid, comment_count - 1, function(err, result) {
                            if (err) {
                                callback(err, null);
                            }
                        });

                        // 更新评论的子评论列表
                        if (parent_cid) {
                            forumCommentService.getCommentById(parent_cid, function(err, result) {
                                if (err) {
                                    console.error(err);
                                    callback(err, null);
                                } else {
                                    var commentList = result.comments;
                                    result.comments = forumCommentService.remove(commentList, cid);
                                    forumCommentService.updateCommentById(parent_cid, result, function(err, result) {
                                        if (err) {
                                            res.json(returnCode.BUSY);
                                        } else {
                                            res.json(returnCode.SUCCESS);
                                        }
                                    });
                                }

                            })

                        };

                    }
                } else {
                    res.json(returnCode.SUCCESS);
                }

            }
        });
    } else {
        res.json(returnCode.WRONG_PARAM);
    }
});


// 更新回复评论
router.post('/thread/comment/update/:cid/comments', function(req, res, next) {
    var tid = req.body.tid || null;
    var cid = req.params.cid || null;
    var replayComment_id = req.body.replayComment_id;
    // 获取评论的子评论列表
    // 更新文章评论 + 1

    forumThreadService.updateThreadCommentCountInc(tid, function(err, thread) {
        if (err) {
            console.error(err);
        }
    });
    // 更新评论对象
    //  result.comment_count+=1;                // 回复评论+1
    forumCommentService.updateCommentById(cid, {$push:{comments:replayComment_id},$inc:{comment_count:1}}, function(err, result) {
        if (err) {
            console.error(err);
            res.json(returnCode.BUSY);
        } else {
            res.json(returnCode.SUCCESS);
        }
    });
});

// 更新评论的子评论列表
router.post('/thread/comment/update/:cid/comments', function(req, res, next) {
    var cid = req.params.cid || null;
    var replayComment_id = req.body.replayComment_id;
    // 获取评论的子评论列表
    forumCommentService.getCommentById(cid, function(err, result) {
        if (err) {
            console.error(err);
            callback(err, null);
        } else {
            var commentList = result.comments;


            result.comments.push(replayComment_id);

            forumCommentService.updateCommentById(cid, result, function(err, result) {
                if (err) {
                    res.json(returnCode.BUSY);
                } else {
                    res.json(returnCode.SUCCESS);
                }
            });

        }
    });
});

//添加评论
router.post('/thread/comment/add', function(req, res, next) {
    var tid = req.body.tid || null;
    var entity = req.body;
    var ip = getClientIP(req);
    entity.created = new Date();
    entity.ent_code = req.session.user.ent_code;
    entity.ip = ip;


    var uid = req.session.user.id;


    // 正式环境
    if (tid) {
        async.waterfall([
            function(callback) {
                forumUserService.getUserByUid(uid, callback);
            }
        ], function(err, user) {
            if (err) {
                res.json(returnCode.BUSY);
            } else {
                if (user) {
                    entity.from = user._id;
                    // 添加评论
                    forumCommentService.createComment(entity, function(err, result) {
                        if (err) {
                            res.json(returnCode.BUSY);
                        } else {
                            var returnData = {
                                comment: 　result,
                                errorcode: 0,
                                errormsg: '请求成功'
                            }
                            res.json(returnData);
                        }
                    });
                } else {
                    var userentity = {
                        uid: uid,
                        nickName: req.session.user.name,
                        icon: req.session.user.headPic
                    };
                    forumUserService.createUser(userentity, function(err, doc) {
                        if (err) {
                            res.json(returnCode.BUSY);
                        } else {
                            entity.from = doc._id;
                            // 添加评论
                            forumCommentService.createComment(entity, function(err, result) {
                                if (err) {
                                    res.json(returnCode.BUSY);
                                } else {
                                    var returnData = {
                                        comment: 　result,
                                        errorcode: 0,
                                        errormsg: '请求成功'
                                    }
                                    res.json(returnData);
                                }
                            });
                        }
                    });
                }
            }
        });
    } else {
        res.json(returnCode.WRONG_PARAM);
    }

   

    // 测试环境
    // entity.from = '55015675868b65a028187c49';
    // if (tid) {
    //     // 添加评论
    //     forumCommentService.createComment(entity, function(err, result) {
    //         if (err) {
    //             res.json(returnCode.BUSY);
    //         } else {
    //             var returnData = {
    //                 comment: 　result,
    //                 errorcode: 0,
    //                 errormsg: '请求成功'
    //             }
    //             res.json(returnData);
    //         }
    //     });
    // }
});


// 更新文章评论
router.post('/thread/:tid/updateComments', function(req, res, next) {
    var tid = req.params.tid || null;
    var cid = req.body.cid || null;
    // 获取评论的子评论列表
    forumThreadService.getById(tid, function(err, result) {
        if (err) {
            console.error(err);
            callback(err, null);
        } else {
            result.comments.push(cid);
            result.comment_count += 1;

            forumThreadService.updateThreadById(tid, result, function(err, result) {
                if (err) {
                    console.error(err);
                    callback(err, null);
                } else {
                    res.json(returnCode.SUCCESS);
                }
            });
        }
    });

});



var getClientIP = function(req) {
    var ipAddress;
    var headers = req.headers;
    var forwardedIpsStr = headers['x-real-ip'] || headers['x-forwarded-for'];
    if (forwardedIpsStr) {
        ipAddress = forwardedIpsStr;
    } else {
        ipAddress = null;
    }
    if (!ipAddress) {
        ipAddress = req.connection.remoteAddress;
    }
    return ipAddress;
};
